
import os
import typing

import jk_simpleexec
import jk_logging
import jk_utils

from .SpoolInfo import SpoolInfo






class Spooler(object):

	################################################################################################################################
	## Static Helper Methods
	################################################################################################################################

	@staticmethod
	def _copy(inFilePath:str, outFilePath:str, chModValueI:typing.Union[int,None], terminationCheckMethod):
		assert inFilePath != outFilePath

		with open(inFilePath, "rb") as fin:
			if chModValueI is None:
				with open(outFilePath, "wb") as fout:
					Spooler._spool(fin, fout)
			else:
				fdesc = os.open(outFilePath, os.O_WRONLY | os.O_CREAT | os.O_TRUNC, chModValueI)
				with open(fdesc, "wb") as fout:
					Spooler._spool(fin, fout, terminationCheckMethod)
	#

	@staticmethod
	def _spool(fin, fout, terminationCheckMethod) -> int:
		assert fin
		assert fout

		lastBlockSize = 65536
		totalLength = 0

		for dataChunk in iter(lambda: fin.read(65536), b""):
			if terminationCheckMethod:
				terminationCheckMethod()

			assert lastBlockSize == 65536
			lastBlockSize = len(dataChunk)
			assert lastBlockSize > 0
			totalLength += len(dataChunk)
			fout.write(dataChunk)

		return totalLength
	#

	################################################################################################################################
	## Static Public Methods
	################################################################################################################################

	@staticmethod
	def spoolStream(
		fromStream,
		toStream,
		terminationFlag:typing.Union[jk_utils.TerminationFlag,None],
		) -> int:

		if terminationFlag is not None:
			assert isinstance(terminationFlag, jk_utils.TerminationFlag)
			terminationCheckMethod = terminationFlag.check
		else:
			terminationCheckMethod = None

		totalLength = Spooler._spool(fromStream, toStream, terminationCheckMethod)

		return totalLength
	#

	@staticmethod
	def spoolFile(
		fromFilePath:str,
		toFilePath:str,
		bDeleteOriginal:bool,
		chModValue:typing.Union[int,jk_utils.ChModValue,None],
		terminationFlag:typing.Union[jk_utils.TerminationFlag,None],
		log:jk_logging.AbstractLogger,
		) -> SpoolInfo:

		assert isinstance(fromFilePath, str)
		assert isinstance(toFilePath, str)
		assert isinstance(bDeleteOriginal, bool)
		if chModValue is not None:
			if isinstance(chModValue, int):
				chModValueI = chModValue
			else:
				assert isinstance(chModValue, jk_utils.ChModValue)
				chModValueI = int(chModValue)
		if terminationFlag is not None:
			assert isinstance(terminationFlag, jk_utils.TerminationFlag)
			terminationCheckMethod = terminationFlag.check
		else:
			terminationCheckMethod = None
		assert isinstance(log, jk_logging.AbstractLogger)

		# ----

		with log.descend("Copying " + repr(fromFilePath) + " ...") as log2:
			fromFilePath = os.path.abspath(fromFilePath)
			assert os.path.isfile(fromFilePath)

			orgFileSize = os.path.getsize(fromFilePath)

			# TODO: check if target file already exists

			Spooler._copy(fromFilePath, toFilePath, chModValueI, terminationCheckMethod)

			resultFileSize = os.path.getsize(toFilePath)

			if bDeleteOriginal:
				if os.path.isfile(fromFilePath):
					os.unlink(fromFilePath)
			else:
				if not os.path.isfile(fromFilePath):
					raise Exception("Implementation error!")

			return SpoolInfo(fromFilePath, toFilePath, None, None, orgFileSize, resultFileSize)
	#

#


























import os
import typing
import tarfile
import gzip
import bz2
import lzma

import jk_simpleexec
import jk_logging
import jk_utils

from .Spooler import Spooler
from .SpoolInfo import SpoolInfo





class Unpacker(object):

	_TAR_PATH = "/bin/tar"

	################################################################################################################################
	## Static Helper Methods
	################################################################################################################################

	@staticmethod
	def _uncompressGZip(
			inFilePath:str,
			outFilePath:str,
			terminationFlag:typing.Union[jk_utils.TerminationFlag,None]
		):
		assert inFilePath != outFilePath

		with gzip.open(inFilePath, "rb") as fin:
			with open(outFilePath, "wb") as fout:
				Spooler.spoolStream(fin, fout, terminationFlag)
	#

	@staticmethod
	def _uncompressBZip2(
			inFilePath:str,
			outFilePath:str,
			terminationFlag:typing.Union[jk_utils.TerminationFlag,None]
		):

		assert inFilePath != outFilePath

		with bz2.open(inFilePath, "rb") as fin:
			with open(outFilePath, "wb") as fout:
				Spooler.spoolStream(fin, fout, terminationFlag)
	#

	@staticmethod
	def _uncompressXZ(
			inFilePath:str,
			outFilePath:str,
			terminationFlag:typing.Union[jk_utils.TerminationFlag,None]
		):

		assert inFilePath != outFilePath

		with lzma.open(inFilePath, "rb") as fin:
			with open(outFilePath, "wb") as fout:
				Spooler.spoolStream(fin, fout, terminationFlag)
	#

	################################################################################################################################
	## Static Public Methods
	################################################################################################################################

	#
	# Untar the specified <c>tar</c> archive and write the contents to the specified directory.
	#
	# @param	str srcTarFilePath					(required) The tar archive to read
	# @param	str destDirFilePath					(required) The target directory to unpack all data to
	# @param	AbstractLogger log					(required) A logger to write log information to
	#
	@staticmethod
	def untarToDir(
			srcTarFilePath:str,
			destDirPath:str,
			log:jk_logging.AbstractLogger
		) -> None:

		assert isinstance(srcTarFilePath, str)
		assert os.path.isfile(srcTarFilePath)
		assert isinstance(destDirPath, str)
		assert isinstance(log, jk_logging.AbstractLogger)

		# ----

		with log.descend("Unpacking " + repr(srcTarFilePath) + " ...") as log2:
			srcTarFilePath = os.path.abspath(srcTarFilePath)
			assert os.path.isfile(srcTarFilePath)
			destDirPath = os.path.abspath(destDirPath)
			if not os.path.isdir(destDirPath):
				os.makedirs(destDirPath)

			if not os.path.isfile(Unpacker._TAR_PATH):
				raise Exception("'tar' not found!")

			tarArgs = [
				"-xf", srcTarFilePath, "-C", destDirPath
			]
			log2.notice("Invoking /bin/tar with: " + str(tarArgs))
			cmdResult = jk_simpleexec.invokeCmd(Unpacker._TAR_PATH, tarArgs, workingDirectory=destDirPath)

			if cmdResult.returnCode != 0:
				cmdResult.dump(writeFunction=log2.error)
				raise Exception("Failed to run 'tar'!")
	#

	@staticmethod
	def guessCompressionFromFilePath(
			filePath:str
		) -> typing.Union[str,str]:

		assert isinstance(filePath, str)

		if filePath.endswith(".gz"):
			return "gz", filePath[:-3]
		elif filePath.endswith(".bz2"):
			return "bz2", filePath[:-4]
		elif filePath.endswith(".xz"):
			return "xz", filePath[:-3]
		elif filePath.endswith(".tgz"):
			return "gz", filePath[:-2] + "ar"
		elif filePath.endswith(".tbz2"):
			return "bz2", filePath[:-3] + "ar"
		elif filePath.endswith(".txz"):
			return "xz", filePath[:-2] + "ar"
		else:
			raise Exception("Can't guess compression!")
	#

	@staticmethod
	def uncompressFile(
			filePath:str,
			toFilePath:typing.Union[str,None],
			bDeleteOriginal:bool,
			terminationFlag:typing.Union[jk_utils.TerminationFlag,None],
			log:jk_logging.AbstractLogger
		) -> str:

		assert isinstance(filePath, str)
		if toFilePath is not None:
			assert isinstance(toFilePath, str)
		assert isinstance(bDeleteOriginal, bool)
		assert isinstance(log, jk_logging.AbstractLogger)

		# ----

		with log.descend("Uncompressing " + repr(filePath) + " ...") as log2:
			filePath = os.path.abspath(filePath)
			assert os.path.isfile(filePath)

			compression, toFilePath2 = Unpacker.guessCompressionFromFilePath(filePath)
			if toFilePath is None:
				toFilePath = toFilePath2

			if compression in [ "gz", "gzip" ]:
				name = "gzip"
				m = Unpacker._uncompressGZip
			elif compression in [ "bz2", "bzip2" ]:
				name = "bzip2"
				m = Unpacker._uncompressBZip2
			elif compression in [ "xz" ]:
				name = "xz"
				m = Unpacker._uncompressXZ
			else:
				raise Exception("Unknown compression: " + repr(compression))

			log.notice("Unpacking with " + name + " ...")

			orgFileSize = os.path.getsize(filePath)

			# TODO: check if target file already exists

			m(filePath, toFilePath, terminationFlag)

			resultFileSize = os.path.getsize(toFilePath)
			uncompressionFactor = round(100 * orgFileSize / resultFileSize, 2)
			log.notice("Uncompression factor: {}%".format(uncompressionFactor))

			if bDeleteOriginal:
				if os.path.isfile(filePath):
					os.unlink(filePath)
			else:
				if not os.path.isfile(filePath):
					raise Exception("Implementation error!")

			return toFilePath
	#

#


























from genericpath import samestat
import os
import typing
import tarfile
import gzip
import bz2
import lzma

import jk_simpleexec
import jk_logging
import jk_utils

from .Spooler import Spooler
from .SpoolInfo import SpoolInfo






class Packer(object):

	_TAR_PATH = "/bin/tar"

	################################################################################################################################
	## Static Helper Methods
	################################################################################################################################

	@staticmethod
	def _compressGZip(
			inFilePath:str,
			outFilePath:str,
			chModValueI:int = None,
			terminationFlag:jk_utils.TerminationFlag = None
		):

		assert inFilePath != outFilePath

		with open(inFilePath, "rb") as fin:
			if chModValueI is None:
				with gzip.open(outFilePath, "wb") as fout:
					Spooler.spoolStream(fin, fout, terminationFlag)
			else:
				fdesc = os.open(outFilePath, os.O_WRONLY | os.O_CREAT | os.O_TRUNC, chModValueI)
				with open(fdesc, "wb") as fout2:
					with gzip.open(fout2, "wb") as fout:
						Spooler.spoolStream(fin, fout, terminationFlag)
	#

	@staticmethod
	def _compressBZip2(
			inFilePath:str,
			outFilePath:str,
			chModValueI:int = None,
			terminationFlag:jk_utils.TerminationFlag = None
		):

		assert inFilePath != outFilePath

		with open(inFilePath, "rb") as fin:
			if chModValueI is None:
				with bz2.open(outFilePath, "wb") as fout:
					Spooler.spoolStream(fin, fout, terminationFlag)
			else:
				fdesc = os.open(outFilePath, os.O_WRONLY | os.O_CREAT | os.O_TRUNC, chModValueI)
				with open(fdesc, "wb") as fout2:
					with bz2.open(fout2, "wb") as fout:
						Spooler.spoolStream(fin, fout, terminationFlag)
	#

	@staticmethod
	def _compressXZ(
			inFilePath:str,
			outFilePath:str,
			chModValueI:int = None,
			terminationFlag:jk_utils.TerminationFlag = None
		):

		assert inFilePath != outFilePath

		with open(inFilePath, "rb") as fin:
			if chModValueI is None:
				with lzma.open(outFilePath, "wb") as fout:
					Spooler.spoolStream(fin, fout, terminationFlag)
			else:
				fdesc = os.open(outFilePath, os.O_WRONLY | os.O_CREAT | os.O_TRUNC, chModValueI)
				with open(fdesc, "wb") as fout2:
					with lzma.open(fout2, "wb") as fout:
						Spooler.spoolStream(fin, fout, terminationFlag)
	#

	#
	# @return		str name
	# @return		str ext
	# @return		callable m
	#
	@staticmethod
	def _getCompressionParams(compression:str):
		assert isinstance(compression, str)

		if compression in [ "gz", "gzip" ]:
			return "gzip", ".gz", Packer._compressGZip
		elif compression in [ "bz2", "bzip2" ]:
			return "bzip2", ".bz2", Packer._compressBZip2
		elif compression in [ "xz" ]:
			return "xz", ".xz", Packer._compressXZ
		else:
			raise Exception("Unknown compression: " + repr(compression))
	#

	################################################################################################################################
	## Static Public Methods
	################################################################################################################################

	#
	# Pack the specified directory in a tar file.
	#
	# @param		str srcDirPath						(required) The directory to pack
	# @param		str destTarFilePath					(required) The tar file to create
	# @param		AbstractLogger log					(required) A logger to write log information to
	#
	@staticmethod
	def tarDir(
			srcDirPath:str,
			destTarFilePath:str,
			log:jk_logging.AbstractLogger
		):

		assert isinstance(srcDirPath, str)
		assert isinstance(destTarFilePath, str)
		assert isinstance(log, jk_logging.AbstractLogger)

		# ----

		with log.descend("Packing " + repr(srcDirPath) + " ...") as log2:
			srcDirPath = os.path.abspath(srcDirPath)
			assert os.path.isdir(srcDirPath)
			destTarFilePath = os.path.abspath(destTarFilePath)

			if not os.path.isfile(Packer._TAR_PATH):
				raise Exception("'tar' not found!")

			tarArgs = [
				"-cf", destTarFilePath, "."
			]

			log2.notice("Invoking /bin/tar with: " + str(tarArgs))
			cmdResult = jk_simpleexec.invokeCmd2(
				cmdPath=Packer._TAR_PATH,
				cmdArgs=tarArgs,
				workingDirectory=srcDirPath,
			)

			if cmdResult.returnCode != 0:
				cmdResult.dump(writeFunction=log2.error)
				raise Exception("Failed to run 'tar'!")
	#

	#
	# Pack the contents of the specified directory in a tar file.
	#
	# @param	str srcDirPath						(required) The directory to pack
	# @param	str destTarFilePath					(required) The tar file to create
	# @param	str[] filesAndDirsToInclude			(optional) The file and directorie names (without path!) to include.
	#												If <c>None</c> is specified the source directory is scanned and all
	#												files and directories found there will be included automatically.
	# @param	AbstractLogger log					(required) A logger to write log information to
	#
	@staticmethod
	def tarDirContents(
			srcDirPath:str,
			destTarFilePath:str,
			filesAndDirsToInclude:typing.List[str] = None,
			log:jk_logging.AbstractLogger = None,
		):

		assert isinstance(srcDirPath, str)
		assert isinstance(destTarFilePath, str)
		if filesAndDirsToInclude is not None:
			assert isinstance(filesAndDirsToInclude, (tuple,list))
			for fn in filesAndDirsToInclude:
				assert isinstance(fn, str)
				assert fn
		assert isinstance(log, jk_logging.AbstractLogger)

		# ----

		with log.descend("Packing " + repr(srcDirPath) + " ...") as log2:
			srcDirPath = os.path.abspath(srcDirPath)
			assert os.path.isdir(srcDirPath)
			destTarFilePath = os.path.abspath(destTarFilePath)

			if filesAndDirsToInclude is None:
				filesAndDirsToInclude = [ x.name for x in os.scandir(srcDirPath) ]

			if not os.path.isfile(Packer._TAR_PATH):
				raise Exception("'tar' not found!")

			tarArgs = [
				"-cf", destTarFilePath
			] + filesAndDirsToInclude

			log2.notice("Invoking /bin/tar ...")
			cmdResult = jk_simpleexec.invokeCmd2(
				cmdPath=Packer._TAR_PATH,
				cmdArgs=tarArgs,
				workingDirectory=srcDirPath,
			)

			if cmdResult.returnCode != 0:
				cmdResult.dump(writeFunction=log2.error)
				raise Exception("Failed to run 'tar'!")
	#

	@staticmethod
	@jk_utils.deprecated
	def compressFile(
			filePath:str,
			compression:str,
			bDeleteOriginal:bool,
			terminationFlag:typing.Union[jk_utils.TerminationFlag,None],
			log:jk_logging.AbstractLogger
		) -> str:

		assert isinstance(filePath, str)
		assert isinstance(compression, str)
		assert isinstance(bDeleteOriginal, bool)
		assert isinstance(log, jk_logging.AbstractLogger)

		# ----

		with log.descend("Compressing " + repr(filePath) + " ...") as log2:
			filePath = os.path.abspath(filePath)
			assert os.path.isfile(filePath)

			name, ext, m = Packer._getCompressionParams(compression)

			log.notice("Packing with " + name + " ...")

			orgFileSize = os.path.getsize(filePath)

			toFilePath = filePath + ext

			# TODO: check if target file already exists

			m(filePath, toFilePath, None, terminationFlag)

			resultFileSize = os.path.getsize(toFilePath)
			compressionFactor = round(100 * resultFileSize / orgFileSize, 2)
			log.notice("Compression factor: {}%".format(compressionFactor))

			if bDeleteOriginal:
				if os.path.isfile(filePath):
					os.unlink(filePath)
			else:
				if not os.path.isfile(filePath):
					raise Exception("Implementation error!")

			return toFilePath
	#

	#
	# Compress the specified file.
	#
	# @param	str filePath						(required) The path of the file to compress.
	# @param	str toFilePath						(optional) The path of the file to write the compressed data to.
	#												If <c>None</c> a new file path is created with suitable exension based on the
	#												compression type.
	# @param	str compression						(required) The compression. Valid values are: "gz", "gzip", "bz2", "bzip2", "xz"
	# @param	bool bDeleteOriginal				(required) If <c>True</c> the source file will be deleted after successfull compression.
	# @param	int|str|ChModValue chModValue		(optional) If specified this change-mode value will be used to set the permission of
	#												the created file.
	# @param	TerminationFlag terminationFlag		(optional) A termination flag for graceful asynchroneous termination.
	# @param	AbstractLogger log					(required) A logger to write log information to
	#
	@staticmethod
	def compressFile2(
			filePath:str,
			toFilePath:typing.Union[str,None],
			compression:str,
			bDeleteOriginal:bool,
			chModValue:typing.Union[int,str,jk_utils.ChModValue,None],
			terminationFlag:typing.Union[jk_utils.TerminationFlag,None],
			log:jk_logging.AbstractLogger,
		) -> SpoolInfo:

		assert isinstance(filePath, str)
		if toFilePath is not None:
			assert isinstance(toFilePath, str)
		assert isinstance(compression, str)
		assert isinstance(bDeleteOriginal, bool)
		if chModValue is not None:
			if isinstance(chModValue, int):
				chModValueI = chModValue
			else:
				chModValueI = jk_utils.ChModValue.create(chModValue).toInt()
		assert isinstance(log, jk_logging.AbstractLogger)

		# ----

		with log.descend("Compressing " + repr(filePath) + " ...") as log2:
			filePath = os.path.abspath(filePath)
			assert os.path.isfile(filePath)

			compressionName, compressionFileExt, m = Packer._getCompressionParams(compression)

			log.notice("Packing with " + compressionName + " ...")

			orgFileSize = os.path.getsize(filePath)

			if toFilePath is None:
				toFilePath = filePath + compressionFileExt

			# TODO: check if target file already exists

			m(filePath, toFilePath, chModValueI, terminationFlag)

			resultFileSize = os.path.getsize(toFilePath)

			if bDeleteOriginal:
				if os.path.isfile(filePath):
					os.unlink(filePath)
			else:
				if not os.path.isfile(filePath):
					raise Exception("Implementation error!")

			return SpoolInfo(filePath, toFilePath, compressionName, compressionFileExt, orgFileSize, resultFileSize)
	#

#

























################################################################################
################################################################################
###
###  This file is automatically generated. Do not change this file! Changes
###  will get overwritten! Change the source file for "setup.py" instead.
###  This is either 'packageinfo.json' or 'packageinfo.jsonc'
###
################################################################################
################################################################################


from setuptools import setup

def readme():
	with open("README.md", "r", encoding="UTF-8-sig") as f:
		return f.read()

setup(
	author = "Jürgen Knauth",
	author_email = "pubsrc@binary-overflow.de",
	classifiers = [
		"Development Status :: 5 - Production/Stable",
		"Programming Language :: Python :: 3",
	],
	description = "Helper module to create or unpack tar archives, compress or uncompress files.",
	include_package_data = False,
	install_requires = [
		"jk_simpleexec",
		"jk_logging",
		"jk_prettyprintobj",
	],
	keywords = [
		"gz",
		"gzip",
		"tar",
		"bz2",
		"bzip",
		"xz",
	],
	license = "proprietary",
	name = "jk_packunpack",
	packages = [
		"jk_packunpack",
	],
	version = "0.2022.2.4",
	zip_safe = False,
	long_description = readme(),
	long_description_content_type="text/markdown",
)
